import time
from resetMachine import *

class TestTime:

    def test_sleepMs(self, resetMachine):
        time.sleep_ms(1)
        time.sleep_ms(2)
        time.sleep_ms(3)
        assert machine.expectedTimeSleepMs == [1,2,3]

    def test_sleepUs(self, resetMachine):
        time.sleep_us(4)
        time.sleep_us(5)
        time.sleep_us(6)
        assert machine.expectedTimeSleepUs == [4,5,6]


from resetMachine import *

@pytest.fixture
def outPin():
    machine.resetExpectationsForTesting()
    ledPin = machine.Pin(1, machine.Pin.OUT)
    ledPin.resetExpectationsForTesting()
    return ledPin


class TestPinOut:

    def test_ErrorRaisedWhenGettingValueOfUnitializedPin(self, outPin):
        machine.resetExpectationsForTesting()
        ledPin = machine.Pin(1, machine.Pin.OUT)
        try:
            ledPin.value()
            raise Exception("Expected Exception. None Raised.")
        except Exception as ex:
            assert ex.args[0] == 'Checking Value of Uninitialized OUT Pin.  Set the value before checking.'

    def test_pinOnSetsValueHigh(self, outPin):
        outPin.on()
        assert outPin.value() == 1

    def test_pinOffSetsValueLow(self, outPin):
        outPin.off()
        assert outPin.value() == 0

    def test_pinValueHighSetsValueHigh(self, outPin):
        outPin.value(1)
        assert outPin.value() == 1

    def test_pinValueLowSetsValueLow(self, outPin):
        outPin.value(0)
        assert outPin.value() == 0

    def test_supportsCollectingMultiplePinOutValues(self, outPin):
        outPin.value(0)
        outPin.value(1)
        outPin.value(1)
        assert outPin.triggerValuesForTesting == [0, 1, 1]



from resetMachine import *
import neopixel
import pytest, time

@pytest.fixture()
def tenPixelStrand():
    pin = machine.Pin(5)
    return neopixel.NeoPixel(pin, n=10)


black = (0, 0, 0)
red = (255, 0, 0)
green = (0, 255, 0)


class TestNeoPixel:

    pin = machine.Pin(5)

    def test_canSetPixelColor(self, resetMachine, tenPixelStrand):
        tenPixelStrand[0] = green
        tenPixelStrand[1] = red
        assert tenPixelStrand[0] == green
        assert tenPixelStrand[1] == red

    def test_mustCallWriteToDisplay(self, resetMachine, tenPixelStrand):
        tenPixelStrand[0] = green
        tenPixelStrand[1] = red
        assert len(tenPixelStrand.writesForTesting) == 0
        tenPixelStrand.write()
        assert len(tenPixelStrand.writesForTesting) == 1

    def test_fill(self, resetMachine, tenPixelStrand):
        tenPixelStrand.fill(green)
        assert _allPixelsAreColor(tenPixelStrand, green)

    def test_recordsWrites(self, resetMachine, tenPixelStrand):
        delayTime = 300
        tenPixelStrand.fill(green)
        tenPixelStrand.write()
        time.sleep(delayTime / 1000)
        tenPixelStrand.fill(red)
        tenPixelStrand.write()

        writeHistory = tenPixelStrand.writesForTesting
        assert len(writeHistory) == 2
        assert _allPixelsAreColor(writeHistory[0], green)
        assert writeHistory[0].timeFromFirstWrite == 0
        assert _allPixelsAreColor(writeHistory[1], red)
        assert _approximately(writeHistory[1].timeFromFirstWrite) == delayTime

    def test_writeUpdatesPixels(self, resetMachine, tenPixelStrand):
        tenPixelStrand[0] = green
        tenPixelStrand[1] = red
        tenPixelStrand.write()
        assert len(tenPixelStrand.writesForTesting) == 1
        writtenStrand = tenPixelStrand.writesForTesting[0]
        assert writtenStrand[0] == green
        assert writtenStrand[1] == red
        assert writtenStrand.timeFromFirstWrite == 0

    def test_initWithDefaults(self, resetMachine):
        np = neopixel.NeoPixel(self.pin, n=10)
        assert np.pin == self.pin
        assert np.n == 10
        assert np.bpp == 3
        assert np.timing == 1

    def test_initWithOverrides(self, resetMachine):
        np = neopixel.NeoPixel(self.pin, n=10, bpp=4, timing=2)
        assert np.bpp == 4
        assert np.timing == 2


def _approximately(exactMilliSeconds):
    return int(exactMilliSeconds / 10) * 10

def _allPixelsAreColor(strand, color):
    pixelCount = strand.n
    for  i in range (pixelCount):
        if strand[i] != color:
            return False
    return True



import machine
import pytest
from resetMachine import *

class TestPulse:
    
    def test_returnsExpectedPulse(self, resetMachine):
        machine.expectedPulseTimeForTesting = 123
        assert machine.time_pulse_us(pin=1, pulse_level=1, timeout_us=1000) == 123

    def test_returnsExpectedPulseTimeError(self, resetMachine):
        machine.expectedPulseTimeErrorForTesting = OSError('error for testing')

        try:
            machine.time_pulse_us(pin=1, pulse_level=1, timeout_us=1000)
            raise Exception("Expected error, none raised")
        except OSError as ex:
            assert(ex.args[0] == 'error for testing')

    def test_supportsMultipleExpectedPulseValues(self, resetMachine):
        machine.expectedPulseTimeForTesting = [123, 456, 789]
        assert machine.time_pulse_us(pin=1, pulse_level=1, timeout_us=1000) == 123
        assert machine.time_pulse_us(pin=1, pulse_level=1, timeout_us=1000) == 456
        assert machine.time_pulse_us(pin=1, pulse_level=1, timeout_us=1000) == 789

    def test_handlesAskingForMoreValuesThanGiven(self, resetMachine):
        machine.expectedPulseTimeForTesting = [123, 456, 789]

        while len(machine.expectedPulseTimeForTesting) > 0:
            machine.time_pulse_us(pin=1, pulse_level=1, timeout_us=1000)

        try:
            machine.time_pulse_us(pin=1, pulse_level=1, timeout_us=1000)
            raise Exception("Expected error, none raised")
        except Exception as ex:
            assert ex.args[0] == "unexpected call to time_pulse_us on empty expectation list"

    def test_supportsErrorInExpectedPulseValues(self, resetMachine):
        machine.expectedPulseTimeForTesting = [123, OSError('error for testing'), 789]
        assert machine.time_pulse_us(pin=1, pulse_level=1, timeout_us=1000) == 123

        try:
            machine.time_pulse_us(pin=1, pulse_level=1, timeout_us=1000) == 456
            raise Exception("Expected error, none raised")
        except OSError as ex:
            assert(ex.args[0] == 'error for testing')

        assert machine.time_pulse_us(pin=1, pulse_level=1, timeout_us=1000) == 789

